import 'package:flutter/material.dart';
import 'dart:async';

import './lock_widget.dart';
import './circle.dart';

class LockContainer extends StatefulWidget {
  final String initMessage;
  final String errorMessage;
  final CircleAttribute attribute;
  final LockCompleteCallback completeCallback;

  LockContainer(
      {this.initMessage: '',
      this.errorMessage: '',
      this.attribute: CircleAttribute.normalAttribute,
      this.completeCallback: null});

  @override
  State<StatefulWidget> createState() {
    return _LockContainerState();
  }
}

class _LockContainerState extends State<LockContainer> {
  String _message = '';
  Color _messageColor = Colors.black;

  LockState _lockState = LockState.normal;

  @override
  void initState() {
    super.initState();
    _message = widget.initMessage;
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Center(child: Text(_message, style: TextStyle(color: _messageColor),)),
        SizedBox(
          height: 20.0,
        ),
        LockWidget(
          lockState: _lockState,
          attribute: widget.attribute,
          completeCallback: (String password) {
            if (widget.completeCallback != null) {
              setState(() {
                bool success = widget.completeCallback(password);

                if (success) {
                  _lockState = LockState.normal;
                } else {
                  _lockState = LockState.error;
                  _message = widget.errorMessage;
                  _messageColor = Colors.red;
                  // 开启一个定时器，一定时间后清空已画的手势密码
                  Timer(Duration(seconds: 2), _handleTimer);
                }
              });
            }
          },
        )
      ],
    );
  }

  void _handleTimer() {
    setState(() {
      _lockState = LockState.normal;
    });
  }
}
